[build-system]
requires = ["setuptools>=50.3.2", "wheel", "setuptools-declarative-requirements", "setuptools_scm[toml]>=3.4"]
build-backend = "setuptools.build_meta"

[tool.setuptools_scm]
write_to = "src/saltext/{{ package_name }}/version.py"
write_to_template = "__version__ = \"{version}\""

[tool.black]
line-length = 100
